#################################################################################
##PACKAGE(help) FiniteGroups
##AUTHOR        V. Dabbaghian-Abdoly
##DATE          November 2006
##
##CALLINGSEQ
##- FiniteGroups['command'](arguments)
##- 'command'(arguments)
##
##DESCRIPTION
##
##- The `FiniteGroups` package is a collection of data and routines for creating 
##  finite groups.
##  This contains a permutation representation and a representation by 
##  generators and relations.
## 
##SECTION List of GraphTheory Package Commands
##
##- The following is a list of the commands in the main FiniteGroups package.
##
##-(lead=indent)    "AllSmallGroups"
##-(lead=indent)    "AlternatingGroup"
##-(lead=indent)    "CyclicGroup"
##-(lead=indent)    "DicyclicGroup"
##-(lead=indent)    "DihedralGroup"
##-(lead=indent)    "ExceptionalGroup"
##-(lead=indent)    "ExceptionalGroupOrder"
##-(lead=indent)    "FixedPoints"
##-(lead=indent)    "QuaternionGroupMultiplyMatrix"
##-(lead=indent)    "QuaternionGroup"
##-(lead=indent)    "GeneralLinearGroup"
##-(lead=indent)    "GeneralLinearGroupOrder"
##-(lead=indent)    "GeneralOrthogonalGroup"
##-(lead=indent)    "GeneralOrthogonalGroupOrder"
##-(lead=indent)    "GeneralUnitaryGroup"
##-(lead=indent)    "GeneralUnitaryGroupOrder"
##-(lead=indent)    "LargestMovedPoint"
##-(lead=indent)    "MathieuGroup"
##-(lead=indent)    "MathieuGroupOrder"
##-(lead=indent)    "MetacyclicGroup"
##-(lead=indent)    "MovedPoints"
##-(lead=indent)    "NumSmallGroups"
##-(lead=indent)    "OrthogonalGroup"
##-(lead=indent)    "OrthogonalGroupOrder"
##-(lead=indent)    "ProjectiveGeneralLinearGroup"
##-(lead=indent)    "ProjectiveGeneralLinearGroupOrder"
##-(lead=indent)    "ProjectiveGeneralOrthogonalGroup"
##-(lead=indent)    "ProjectiveGeneralOrthogonalGroupOrder"
##-(lead=indent)    "ProjectiveGeneralUnitaryGroup"
##-(lead=indent)    "ProjectiveGeneralUnitaryGroupOrder"
##-(lead=indent)    "ProjectiveSpecialLinearGroup"
##-(lead=indent)    "ProjectiveSpecialLinearGroupOrder"
##-(lead=indent)    "ProjectiveSpecialOrthogonalGroup"
##-(lead=indent)    "ProjectiveSpecialOrthogonalGroupOrder"
##-(lead=indent)    "ProjectiveSpecialUnitaryGroup"
##-(lead=indent)    "ProjectiveSpecialUnitaryGroupOrder"
##-(lead=indent)    "ProjectiveSymplecticGroup"
##-(lead=indent)    "ProjectiveSymplecticGroupOrder"
##-(lead=indent)    "SmallGroup"
##-(lead=indent)    "SamllestMovedPoint"
##-(lead=indent)    "SpecialLinearGroup"
##-(lead=indent)    "SpecialLinearGroupOrder"
##-(lead=indent)    "SpecialOrthogonalGroup"
##-(lead=indent)    "SpecialOrthogonalGroupOrder"
##-(lead=indent)    "SpecialUnitaryGroup"
##-(lead=indent)    "SpecialUnitaryGroupOrder"
##-(lead=indent)    "SporadicGroup"
##-(lead=indent)    "SporadicGroupOrder"
##-(lead=indent)    "SymmetricGroup"
##-(lead=indent)    "SymplecticGroup"
##-(lead=indent)    "SymplecticGroupOrder"
##
##SECTION Accessing the FiniteGroups Package Commands
##
##- Each command in the `FiniteGroups` package can be accessed by using either
##  the "long form" or the "short form" of the command name in the command
##  calling sequence.  For example, if ~n~ is a number you may use either
##  ~FiniteGroups[CyclicGroup](n)~ or ~with(FiniteGroups);~ then ~CyclicGroup(n)~.
##
##- Because the underlying implementation of the `FiniteGroups` package is a
##  module, it is possible to use the form ~FiniteGroups:-command~ to access a
##  command from the package. For more information, see "Module Members".
##
##NOTES
##
##EXAMPLES
##
##> with(FiniteGroups):
##
##- The general linear group of dimension 3 on a finite field of order 5.
##
##> G := GeneralLinearGroup(3,5);
##
##- Number of all non-isomorphic small groups of order 96.
##
##> H := NumSmallGroups(96);
##
##- The symmetric group on 8 elements.
##
##> N := SymmetricGroup(8);
##
##SEEALSO
##
##- "FiniteGroups examples"
##
##XREFMAP
##-    "FiniteGroups examples"                    : Help:examples[FiniteGroups]
##-    "AllSmallGroups"                           : Help:FiniteGroups[AllSmallGroups]
##-    "AlternatingGroup"                         : Help:FiniteGroups[AlternatingGroup]
##-    "CyclicGroup"                              : Help:FiniteGroups[CyclicGroup]
##-    "DicyclicGroup"                            : Help:FiniteGroups[DicyclicGroup]
##-    "DihedralGroup"                            : Help:FiniteGroups[DihedralGroup]
##-    "FixedPoints"                              : Help:FiniteGroups[FixedPoints]
##-    "ExceptionalGroup"                         : Help:FiniteGroups[ExceptionalGroup]
##-    "ExceptionalGroupOrder"                    : Help:FiniteGroups[ExceptionalGroupOrder]
##-    "QuaternionGroupMultiplyMatrix"            : Help:FiniteGroups[QuaternionGroupMultiplyMatrix]
##-    "QuaternionGroup"                          : Help:FiniteGroups[QuaternionGroup]
##-    "GeneralLinearGroup"                       : Help:FiniteGroups[GeneralLinearGroup]
##-    "GeneralLinearGroupOrder"                  : Help:FiniteGroups[GeneralLinearGroupOrder]
##-    "GeneralOrthogonalGroup"                   : Help:FiniteGroups[GeneralOrthogonalGroup]
##-    "GeneralOrthogonalGroupOrder"              : Help:FiniteGroups[GeneralOrthogonalGroupOrder]
##-    "GeneralUnitaryGroup"                      : Help:FiniteGroups[GeneralUnitaryGroup]
##-    "GeneralUnitaryGroupOrder"                 : Help:FiniteGroups[GeneralUnitaryGroupOrder]
##-    "LargestMovedPoint"                        : Help:FiniteGroups[LargestMovedPoint]
##-    "MathieuGroup"                             : Help:FiniteGroups[MathieuGroup]
##-    "MathieuGroupOrder"                        : Help:FiniteGroups[MathieuGroupOrder]
##-    "MetacyclicGroup"                          : Help:FiniteGroups[MetacyclicGroup]
##-    "MovedPoints"                              : Help:FiniteGroups[MovedPoints]
##-    "NumSmallGroups"                           : Help:FiniteGroups[NumSmallGroups]
##-    "OrthogonalGroup"                          : Help:FiniteGroups[OrthogonalGroup]
##-    "OrthogonalGroupOrder"                     : Help:FiniteGroups[OrthogonalGroupOrder]
##-    "ProjectiveGeneralLinearGroup"             : Help:FiniteGroups[ProjectiveGeneralLinearGroup]
##-    "ProjectiveGeneralLinearGroupOrder"        : Help:FiniteGroups[ProjectiveGeneralLinearGroupOrder]
##-    "ProjectiveGeneralOrthogonalGroup"         : Help:FiniteGroups[ProjectiveGeneralOrthogonalGroup]
##-    "ProjectiveGeneralOrthogonalGroupOrder"    : Help:FiniteGroups[ProjectiveGeneralOrthogonalGroupOrder]
##-    "ProjectiveGeneralUnitaryGroup"            : Help:FiniteGroups[ProjectiveGeneralUnitaryGroup]
##-    "ProjectiveGeneralUnitaryGroupOrder"       : Help:FiniteGroups[ProjectiveGeneralUnitaryGroupOrder]
##-    "ProjectiveSpecialLinearGroup"             : Help:FiniteGroups[ProjectiveSpecialLinearGroup]
##-    "ProjectiveSpecialLinearGroupOrder"        : Help:FiniteGroups[ProjectiveSpecialLinearGroupOrder]
##-    "ProjectiveSpecialOrthogonalGroup"         : Help:FiniteGroups[ProjectiveSpecialOrthogonalGroup]
##-    "ProjectiveSpecialOrthogonalGroupOrder"    : Help:FiniteGroups[ProjectiveSpecialOrthogonalGroupOrder]
##-    "ProjectiveSpecialUnitaryGroup"            : Help:FiniteGroups[ProjectiveSpecialUnitaryGroup]
##-    "ProjectiveSpecialUnitaryGroupOrder"       : Help:FiniteGroups[ProjectiveSpecialUnitaryGroupOrder]
##-    "ProjectiveSymplecticGroup"                : Help:FiniteGroups[ProjectiveSymplecticGroup]
##-    "ProjectiveSymplecticGroupOrder"           : Help:FiniteGroups[ProjectiveSymplecticGroupOrder]
##-    "SmallGroup"                               : Help:FiniteGroups[SmallGroup]
##-    "SmallestMovedPoint"                       : Help:FiniteGroups[SmallestMovedPoint]
##-    "SpecialLinearGroup"                       : Help:FiniteGroups[SpecialLinearGroup]
##-    "SpecialLinearGroupOrder"                  : Help:FiniteGroups[SpecialLinearGroupOrder]
##-    "SpecialOrthogonalGroup"                   : Help:FiniteGroups[SpecialOrthogonalGroup]
##-    "SpecialOrthogonalGroupOrder"              : Help:FiniteGroups[SpecialOrthogonalGroupOrder]
##-    "SpecialUnitaryGroup"                      : Help:FiniteGroups[SpecialUnitaryGroup]
##-    "SpecialUnitaryGroupOrder"                 : Help:FiniteGroups[SpecialUnitaryGroupOrder]
##-    "SporadicGroup"                            : Help:FiniteGroups[SporadicGroup]
##-    "SporadicGroupOrder"                       : Help:FiniteGroups[SporadicGroupOrder]
##-    "SymmetricGroup"                           : Help:FiniteGroups[SymmetricGroup]
##-    "SymplecticGroup"                          : Help:FiniteGroups[SymplecticGroup]
##-    "SymplecticGroupOrder"                     : Help:FiniteGroups[SymplecticGroupOrder]
#
# FiniteGroups Package: Version 1.1, December, 2006.
# Authors:
#    Vahid Dabbaghian-Abdoly : vdabbagh@cecm.sfu.ca
#    Greg Fee                : gfee@cecm.sfu.ca

#####################
# GLOBAL Procedures #
#####################

`type/group` := specfunc(anything, {permgroup, grelgroup}):
`type/grelgroup` := specfunc(anything, grelgroup):
`type/permgroup` := specfunc(anything, permgroup):

permgroupShortPrintSize:=15:
grelgroupShortPrintSize:=300:

`print/permgroup` := proc(p1,p2)
  global permgroupShortPrintSize;
  if p1>permgroupShortPrintSize then
  cat(`permgroup of degree `,p1,` with `,nops(p2),` generators`);
  end if;
 end proc:

`print/grelgroup` := proc(p1,p2)
 global grelgroupShortPrintSize;
  if length(p1)+length(p2)>grelgroupShortPrintSize then
  cat(`grelgroup on `,nops(p1),` symbols with `,nops(p2),` relations`);
  end if;
 end proc:

#savelib('`type/group`', '`type/grelgroup`', '`type/permgroup`', \
     '`print/permgroup`', '`print/grelgroup`', \
     'permgroupShortPrintSize', 'grelgroupShortPrintSize');

############################
# End Of GLOBAL Procedures #
############################

FiniteGroups := module()

export
     AllSmallGroups, 
     AlternatingGroup, 
     CyclicGroup, 
     DicyclicGroup, 
     DihedralGroup, 
     ExceptionalGroup, 
     ExceptionalGroupOrder,
     FixedPoints, 
     QuaternionGroupMultiplyMatrix, 
     QuaternionGroup, 
     GeneralLinearGroup, 
     GeneralLinearGroupOrder,
     GeneralOrthogonalGroup, 
     GeneralOrthogonalGroupOrder,
     GeneralUnitaryGroup,
     GeneralUnitaryGroupOrder,
     IsomorphismPermGroup, 
     LargestMovedPoint, 
     MathieuGroup, 
     MathieuGroupOrder,
     MetacyclicGroup, 
     MovedPoints, 
     NumSmallGroups, 
     OrthogonalGroup, 
     OrthogonalGroupOrder,
     ProjectiveGeneralLinearGroup, 
     ProjectiveGeneralLinearGroupOrder, 
     ProjectiveGeneralUnitaryGroup, 
     ProjectiveGeneralUnitaryGroupOrder,
     ProjectiveSpecialLinearGroup, 
     ProjectiveSpecialLinearGroupOrder,
     ProjectiveSpecialUnitaryGroup,
     ProjectiveSpecialUnitaryGroupOrder,
     ProjectiveSymplecticGroup, 
     ProjectiveSymplecticGroupOrder,
     SmallGroup, 
     SmallestMovedPoint, 
     SpecialLinearGroup, 
     SpecialLinearGroupOrder,
     SpecialOrthogonalGroup, 
     SpecialOrthogonalGroupOrder,
     SpecialUnitaryGroup, 
     SpecialUnitaryGroupOrder,
     SporadicGroup, 
     SporadicGroupOrder,
     SymmetricGroup, 
     SymplecticGroup,
     SymplecticGroupOrder,
     ExceptionalGroups, 
     GeneralLinearGroups, 
     GeneralOrthogonalGroups, 
     GeneralUnitaryGroups,
     MathieuGroups, 
     OrthogonalGroups, 
     ProjectiveGeneralLinearGroups,  
     ProjectiveGeneralUnitaryGroups, 
     ProjectiveSpecialLinearGroups, 
     ProjectiveSpecialUnitaryGroups, 
     ProjectiveSymplecticGroups, 
     SmallGroups, 
     SpecialLinearGroups, 
     SpecialOrthogonalGroups, 
     SpecialUnitaryGroups, 
     SporadicGroups, 
     SymplecticGroups;

local
     AllSmallGrelgroups,
     AllSmallPermgroups,
     SmallGrelgroup,
     SmallPermgroup,
     QuaternionGroupGrelgroup,
     QuaternionGroupPermgroup,
     MetacyclicGrelgroup,
     MetacyclicPermgroup,
     DicyclicGrelgroup,
     DicyclicPermgroup,
     DihedralGrelgroup,
     DihedralPermgroup,
     CyclicGrelgroup,
     CyclicPermgroup,
     IsPrimePower;
     
option package;


IsPrimePower := proc(q::posint)
local f;
f := ifactors(q);
if nops(op(2,f)) > 1 then
   false;
else true;
end if;
end;


############################################################################
##PROCEDURE(doti)	FiniteGroups[GeneralLinearGroup]
##TITLE                 GeneralLinearGroup
##TITLE                 GeneralLinearGroupOrder
##ALIAS			FiniteGroups[GeneralLinearGroup], FiniteGroups[GeneralLinearGroupOrder]
##CALLINGSEQ
##- GeneralLinearGroup('n', 'q')
##- GeneralLinearGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the general linear group GL(n,q).
##
##DESCRIPTION
##- The general linear group is the set of all n x n nonsingular matrices over 
##  a finite field of size q. 
##  The range for n and q are as follows,
##        q<= 100 for n=2,
##        q<= 20  for n=3,
##        q<= 10  for n=4,
##        q<= 5   for n=5,
##        q = 2   for n=6, 7, 8, 9, 10.
##
##EXAMPLES
##> with(FiniteGroups):
##> GeneralLinearGroup(2,3);
##<   permgroup(8, {[[1, 2], [3, 5], [4, 7]], [[1, 3, 6], [2, 4, 8]]})
##> GeneralLinearGroup(4,3);
#<             permgroup of degree 80 with 2 generators
##< permgroup(80,{[[1, 2], [3, 5], [4, 7], [6, 10], [8, 13], [9, 15], [11, 18], [12, 20], [14, 23], [16, 26], [17, 27], [21, 28], [22, 29], [25, 33], [30, 39], [31, 41], [32, 43], [40, 50], [44, 53], [45, 46], [47, 48], [55, 63], [57, 65], [59, 66], [61, 64], [67, 71], [68, 73]], [[1, 3, 6, 11, 19, 30, 40, 51, 60, 67, 72, 77, 79], [2, 4, 8, 14, 24, 32, 44, 54, 62, 68, 74, 78, 80], [5, 9, 16, 18, 29, 38, 48, 58, 66, 63, 69 , 75, 71], [7, 12, 21, 23, 27, 36, 46, 56, 64, 65, 70, 76, 73], [10, 17, 28, 37, 47, 57, 15, 25, 34, 39, 49, 59, 50], [13, 22, 26, 35, 45, 55, 20, 31, 42, 43, 52, 61, 53]]})
##> GeneralLinearGroupOrder(2,13);
##< 26208
##
##SEEALSO
##- "ProjectiveGeneralLinearGroup"
##- "GeneralUnitaryGroup"
##- "GeneralOrthogonalGroup"

GeneralLinearGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
A := traperror( GeneralLinearGroups[gn] );
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

GeneralLinearGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q 
fi; 
product( (q^n-q^k), k=0..n-1 ); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[ProjectiveGeneralLinearGroup]
##TITLE                 ProjectiveGeneralLinearGroup
##TITLE                 ProjectiveGeneralLinearGroupOrder
##ALIAS			FiniteGroups[ProjectiveGeneralLinearGroup], FiniteGroups[ProjectiveGeneralLinearGroupOrder]
##CALLINGSEQ
##- ProgectiveGeneralLinearGroup('n', 'q')
##- ProgectiveGeneralLinearGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the projective general linear group PGL(n,q).
##
##DESCRIPTION
##- The projective general linear group is the set of all n x n nonsingular 
##  matrices over the field with q elements, modulo the centre. 
##  The range for n and q are as follows,
##        q<= 100 for n=2,
##        q<= 20  for n=3,
##        q<= 10  for n=4,
##        q<= 5   for n=5,
##        q = 2   for n=6, 7, 8, 9, 10.
##
##EXAMPLES
##> with(FiniteGroups):
##> ProjectiveGeneralLinearGroup(2,49);
#!<   permgroup of degree 50 with 2 generators
##<   permgroup(50,{[[3,50,49,48,47,46,45,44,8,43,42,41,40,39,38,37,7,36,35,34,33,32,31,30,6,29,28,27,26,25,24,23,5,22,21,20,19,18,17,16,4,15,14,13,12,11,10,9]],[[1,2,6],[3,4,8],[9,25,33],[10,20,37],[11,32,24],[12,15,41],[13,38,16],[14,30,23],[17,31,19],[18,44,47],[21,46,43],[22,39,49],[26,34,50],[27,48,35],[28,42,40],[29,45,36]]})
##> ProjectiveGeneralLinearGroup(5,2);
#!<   permgroup of degree 31 with 2 generators
##< permgroup(31,{[[16, 24], [17, 25], [18, 26], [19, 27], [20, 28], [21, 29], [22, 30], [23, 31]], [[1, 2, 4 , 8, 16], [3, 6, 12, 24, 17], [5, 10, 20, 9, 18], [7, 14, 28, 25, 19], [11, 22, 13, 26, 21], [15, 30, 29, 27, 23]]})
##> ProjectiveGeneralLinearGroupOrder(4,3);
##< 12130560
##
##SEEALSO
##- "GeneralLinearGroup"

ProjectiveGeneralLinearGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
A := traperror( ProjectiveGeneralLinearGroups[gn] );
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

ProjectiveGeneralLinearGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
(product( (q^n-q^k), k=0..n-1 ))/(q-1); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[SpecialLinearGroup]
##TITLE                 SpecialLinearGroup
##TITLE                 SpecialLinearGroupOrder
##ALIAS			FiniteGroups[SpecialLinearGroup], FiniteGroups[SpecialLinearGroupOrder]
##CALLINGSEQ
##- SpecialLinearGroup('n', 'q')
##- SpecialLinearGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the special linear group SL(n,q).
##
##DESCRIPTION
##- The special linear group is the set of all n x n nonsingular matrices over 
##  a finite field of size q whose the determinant is 1. 
##  The range for n and q are as follows, 
##        q<= 100  for n=2,
##        q<= 20   for n=3,
##        q<= 10   for n=4,
##        q<= 5    for n=5,
##        q = 2    for n=6, 7, 8, 9, 10. 
##
##EXAMPLES
##> with(FiniteGroups):
##> SpecialLinearGroup(3,2);
##< permgroup(7, {[[2, 3], [4, 6]], [[1, 2, 4], [3, 5, 7]]})
##> SpecialLinearGroupOrder(2,3);
##< 24 
##
##SEEALSO
##- "ProjectiveSpecialLinearGroup"
##- "SpecialUnitaryGroup"

SpecialLinearGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
A := traperror( SpecialLinearGroups[gn] ):
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

SpecialLinearGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
(q^(n*(n-1)/2))*(product( (q^(k+1)-1), k=1..n-1) ); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[ProjectiveSpecialLinearGroup]
##TITLE                 ProjectiveSpecialLinearGroup
##TITLE                 ProjectiveSpecialLinearGroupOrder
##ALIAS			FiniteGroups[ProjectiveSpecialLinearGroup], FiniteGroups[ProjectiveSpecialLinearGroupOrder]
##CALLINGSEQ
##- ProjectiveSpecialLinearGroup('n', 'q')
##- ProjectiveSpecialLinearGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the projective special linear group PSL(n,q).
##
##DESCRIPTION
##- The projective special linear group is the set of all n x n nonsingular  
##  matrices over a finite field of size q whose the determinant is the 
##  identity of the field, modulo the centre. 
##  The rang for n and q are as follows,
##        q<= 241 for n=2,
##        q<= 20  for n=3,
##        q<= 10  for n=4,
##        q<= 5   for n=5,
##        q = 2   for n=6, 7, 8, 9, 10.
##
##EXAMPLES
##> with(FiniteGroups):
##> ProjectiveSpecialLinearGroup(3,2);
##<  permgroup(7, {[[4, 6], [5, 7]], [[1, 2, 4], [3, 6, 5]]})
##> ProjectiveSpecialLinearGroup(3,4);
#!< permgroup of degree 21 with 2 generators
##< permgroup(21,{[[3, 4, 5], [7, 9, 8], [10, 14, 18], [11, 17, 20], [12, 15, 21], [13, 16, 19]], [[1, 2, 6, 7, 11, 3, 10], [4, 14, 8, 15, 16, 20, 13], [5, 18, 9, 19, 21, 17, 12]]})
##> ProjectiveSpecialLinearGroupOrder(3,3);
##< 5616 
##
##SEEALSO
##- "SpecialLinearGroup"
##- "ProjectiveSpecialUnitaryGroup"

ProjectiveSpecialLinearGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
A := traperror( ProjectiveSpecialLinearGroups[gn] );
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

ProjectiveSpecialLinearGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
(1/gcd(n,q-1))*(q^(n*(n-1)/2))*(product( (q^(k+1)-1), k=1..n-1) ); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[SpecialUnitaryGroup]
##TITLE                 SpecialUnitaryGroup
##TITLE                 SpecialUnitaryGroupOrder
##ALIAS			FiniteGroups[SpecialUnitaryGroup], FiniteGroups[SpecialUnitaryGroupOrder]
##CALLINGSEQ
##- SpecialUnitaryGroup('n', 'q')
##- SpecialUnitaryGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the special unitary group SU(n,q).
##
##DESCRIPTION
##- The special unitary group is the set of all n x n matrices over the field 
##  with q^2 elements whose determinant is 1 and respect a fixed nondegenerate
##  sesquilinear form. Note that for n=2 the groups SU(n,q) and SL(n,q) are 
##  isomorphic.
##  The range for n and q are as follows,      
##        q<= 100  for n=2, 
##        q<= 10   for n=3,
##        q<= 4    for n=4,
##        q<= 3    for n=5,
##        q = 2    for n=6,7.
##
##EXAMPLES
##> with(FiniteGroups):
##> SpecialUnitaryGroup(3,2);
#!< permgroup of degree 27 with 2 generators
##< permgroup(27,{[[2, 3, 5, 9], [4, 7, 13, 22], [6, 11, 19, 26], [8, 15, 17, 20], [10, 18, 25, 24], [12, 21, 27, 23]], [[1, 2, 4, 8, 16, 24, 21, 7, 14, 11, 20, 27], [3, 6, 12, 5, 10, 9, 17, 18, 19, 25, 22, 26], [13, 23, 15]]})
##> SpecialUnitaryGroupOrder(4,2);
##< 25920  
##
##SEEALSO
##- "ProjectiveSpecialUnitaryGroup"
##- "SpecialLinearGroup"

SpecialUnitaryGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
if n = 2 then
     A := traperror( SpecialLinearGroups[gn] ):
else
     A := traperror( SpecialUnitaryGroups[gn] ):
end if;
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

SpecialUnitaryGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
(q^(n*(n-1)/2))*(product( (q^(k+1)-(-1)^(k+1)), k=1..n-1) ); 
end:


############################################################################
##PROCEDURE(doti)	FiniteGroups[ProjectiveSpecialUnitaryGroup]
##TITLE                 ProjectiveSpecialUnitaryGroup
##TITLE                 ProjectiveSpecialUnitaryGroupOrder
##ALIAS			FiniteGroups[ProjectiveSpecialUnitaryGroup], FiniteGroups[ProjectiveSpecialUnitaryGroupOrder]
##CALLINGSEQ
##- ProjectiveSpecialUnitaryGroup('n', 'q')
##- ProjectiveSpecialUnitaryGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the projective special unitary group PSU(n,q).
##
##DESCRIPTION
##- The special unitary group is the set of all n x n matrices over the field 
##  with q^2 elements whose determinant is 1 and respect a fixed nondegenerate
##  sesquilinear form, modulo the centre. Note that for n=2 the groups PSU(n,q) 
##  and PSL(n,q) are isomorphic. 
##  The range for n and q are as follows,
##        q<= 241 for n=2,
##        q<= 16  for n=3,
##        q<= 5   for n=4,
##        q<= 4   for n=5,
##        q<= 3   for n=6,
##        q = 2   for n=7, 8.
##
##EXAMPLES
##> with(FiniteGroups):
##> ProjectiveSpecialUnitaryGroup(3,3);
#!< permgroup of degree 28 with 2 generators
##< permgroup(28,{[[2, 17], [3, 27], [4, 7], [5, 24], [6, 13], [8, 11], [9, 22], [10, 21], [12, 14], [16, 20], [18, 19], [25, 26]], [[1, 3, 8, 17, 18, 5], [2, 25, 9, 28, 23, 7], [4, 10, 22], [11, 12, 26, 24, 27, 15], [13, 19, 16, 21, 20, 14]]})
##> ProjectiveSpecialUnitaryGroupOrder(5,3);
##< 258190571520 
##
##SEEALSO
##- "SpecialUnitaryGroup"
##- "ProjectiveSpecialLinearGroup"

ProjectiveSpecialUnitaryGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
if n = 2 then
     A := traperror( ProjectiveSpecialLinearGroups[gn] ):
else
     A := traperror( ProjectiveSpecialUnitaryGroups[gn] ):
end if;
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

ProjectiveSpecialUnitaryGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
(1/gcd(n,q+1))*(q^(n*(n-1)/2))*(product( (q^(k+1)-(-1)^(k+1)), k=1..n-1) ); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[GeneralUnitaryGroup]
##TITLE                 GeneralUnitaryGroup
##TITLE                 GeneralUnitaryGroupOrder
##ALIAS			FiniteGroups[GeneralUnitaryGroup], FiniteGroups[GeneralUnitaryGroupOrder]
##CALLINGSEQ
##- GeneralUnitaryGroup('n', 'q')
##- GeneralUnitaryGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the general unitary group GU(n,q).
##
##DESCRIPTION
##- The general unitary group of all n x n matrices over the field with q^2 
##  elements that respect a fixed nondegenerate sesquilinear form. 
##  The range for n and q are as follows,
##        q<= 20 for n=2,
##        q<= 5  for n=3,
##        q<= 4  for n=4,
##        q = 2  for n=5, 6.
##
##EXAMPLES
##> with(FiniteGroups):
##> GeneralUnitaryGroup(2,2);
##<   permgroup(9,{[[1,2,4],[3,6,7,9,5,8]],[[1,3],[2,5],[4,7]]})
##> GeneralUnitaryGroup(2,4);
#!< permgroup of degree 75 with 2 generators
##< permgroup(75,{[[1, 2, 4, 8, 15, 27, 43, 41, 57, 70, 75, 26, 38, 54, 68], [3, 6, 12, 16, 29, 47, 61, 20, 35, 56, 64, 74, 58, 52, 50], [5, 10, 19, 28, 45, 63, 60, 32, 51, 69, 24, 22, 39, 59, 65], [7, 14, 25, 30, 49, 67, 72, 18, 33, 53, 62, 11, 21, 37, 46], [9, 17, 31, 44, 13, 23, 40, 48, 66, 42, 36, 34, 55, 71, 73]], [[1, 3, 7], [2, 5, 11], [4, 9, 18], [6, 13, 24], [8, 16, 30], [10, 20, 36], [12, 22, 23], [14, 26, 42], [15, 28, 46], [17, 32, 52], [19, 34, 35], [21, 38, 58], [25, 41, 60], [27, 44, 62], [29, 48, 59], [31, 50, 51], [33, 54, 39], [37, 57, 40], [43, 61, 72], [45, 64, 71], [47, 65, 66], [49, 68, 55], [53, 70, 56], [63, 73, 74], [67, 75, 69]]})
##> GeneralUnitaryGroupOrder(2,16);
##< 69360 
##
##SEEALSO
##- "ProjectiveGeneralUnitaryGroup"
##- "GeneralLinearGroup"

GeneralUnitaryGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
A := traperror( GeneralUnitaryGroups[gn] ):
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

GeneralUnitaryGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                            
   error "second argument must be a prime power but got: %1", q fi; 
(q+1)*(q^(n*(n-1)/2))*(product( (q^(k+1)-(-1)^(k+1)), k=1..n-1) ); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[ProjectiveGeneralUnitaryGroup]
##TITLE                 ProjectiveGeneralUnitaryGroup
##TITLE                 ProjectiveGeneralUnitaryGroupOrder
##ALIAS			FiniteGroups[ProjectiveGeneralUnitaryGroup], FiniteGroups[ProjectiveGeneralUnitaryGroupOrder]
##CALLINGSEQ
##- ProjectiveGeneralUnitaryGroup('n', 'q')
##- ProjectiveGeneralUnitaryGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the projective general unitary group PGU(n,q).
##
##DESCRIPTION
##- The projective general unitary group of those n x n matrices over the field 
##  with q^2 elements that respect a fixed nondegenerate sesquilinear form, modulo 
##  the centre. Note that for n=2 the groups PGU(n,q) and PGL(n,q) are isomorphic.
##  The range for n and q are as follows,
##        q<= 100  for n=2,
##        q<= 5    for n=3,
##        q<= 4    for n=4,
##        q = 2    for n=5, 6.
##
##EXAMPLES
##> with(FiniteGroups):
##> ProjectiveGeneralUnitaryGroup(2,13);
##<   permgroup(14,{[[1,2,9],[3,8,10],[4,5,12],[6,13,14]],[[3,14,13,12,11,10,9,8,7,6,5,4]]})
##> ProjectiveGeneralUnitaryGroupOrder(4,4);
##< 1018368000  
##
##SEEALSO
##- "GeneralUnitaryGroup"
##- "ProjectiveGeneralLinearGroup"

ProjectiveGeneralUnitaryGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
if n = 2 then
     A := traperror( ProjectiveGeneralLinearGroups[gn] ):
else
     A := traperror( ProjectiveGeneralUnitaryGroups[gn] ):
end if;
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

ProjectiveGeneralUnitaryGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
(q^(n*(n-1)/2))*(product( (q^(k+1)-(-1)^(k+1)), k=1..n-1) ); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[SymplecticGroup]
##TITLE                 SymplecticGroup
##TITLE                 SymplecticGroupOrder
##ALIAS			FiniteGroups[SymplecticGroup], FiniteGroups[SymplecticGroupOrder]
##CALLINGSEQ
##- SymplecticGroup('n', 'q')
##- SymplecticGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the symplectic group Sp(n,q).
##
##DESCRIPTION
##- The symplectic group of all n x n matrices over the field with q elements 
##  that respect a fixed nondegenerate symplectic form. Note that for n=2 the 
##  groups Sp(n,q) and SL(n,q) are isomorphic. 
##  The range for n and q are as follows,
##        q<= 100  for n=2,
##        q<= 10   for n=4,
##        q<= 4    for n=6,
##        q = 2    for n=8, 10.
##
##EXAMPLES
##> with(FiniteGroups):
##> SymplecticGroup(4,2);
##<   permgroup(15, {[[2, 3, 5, 7], [4, 6, 9, 13], [8, 11, 12, 15], [10, 14]], [[1, 2, 4, 3], [5, 8, 12, 7], [6, 10, 13, 15], [11, 14]]})
##> SymplecticGroupOrder(6,2);
##< 1451520 
##
##> SymplecticGroup(4,3);
#!< permgroup of degree 80 with 2 generators
##< permgroup(80,{[[1, 2], [3, 5], [4, 7], [6, 10], [8, 13], [9, 15], [11, 18], [12, 20], [14, 19], [16, 24], [17, 26], [21, 31], [22, 33], [23, 35], [25, 32], [27, 40], [28, 42], [29, 44], [30, 46], [34, 51], [36, 53], [37, 54], [38, 55], [39, 50], [43, 60], [45, 62], [47, 56], [48, 59], [49, 64], [57, 66], [58, 65], [61, 63], [69, 73], [71, 75], [74, 78], [76, 77]], [[1, 3, 6, 11, 19, 29, 45, 63, 72], [2, 4, 8, 14, 18, 28, 43, 61, 70], [5, 9, 16, 25, 38, 56, 62, 71, 76], [7, 12, 21, 32, 49, 53, 60, 69, 74], [10, 17, 27, 41, 59, 68, 40, 58, 44], [13, 22, 34, 52, 54, 67, 51, 66, 42], [15, 23, 36], [20, 30, 47], [24, 37, 55], [26, 39, 57], [31, 48, 64], [33, 50, 65], [73, 77, 79], [75, 78, 80]]})
##SEEALSO
##- "ProjectiveSymplecticGroup"
##- "SpecialLinearGroup"

SymplecticGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
if type(n,odd) then
     error"the dimension must be an even number":
end if;
if n = 2 then
     A := traperror( SpecialLinearGroups[gn] ):
else
     A := traperror( SymplecticGroups[gn] ):
end if;
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

SymplecticGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
(q^((n/2)^2))*(product( (q^(2*k)-1), k=1..n/2) ); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[ProjectiveSymplecticGroup]
##TITLE                 ProjectiveSymplecticGroup
##TITLE                 ProjectiveSymplecticGroupOrder
##ALIAS			FiniteGroups[ProjectiveSymplecticGroup], FiniteGroups[ProjectiveSymplecticGroupOrder]
##CALLINGSEQ
##- ProjectiveSymplecticGroup('n', 'q')
##- ProjectiveSymplecticGroupOrder('n', 'q')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the projective symplectic group PSp(n,q).
##
##DESCRIPTION
##- The projective symplectic group of those n x n matrices over the field 
##  with q elements that respect a fixed nondegenerate symplectic form, 
##  modulo the centre. Note that for n=2 the groups PSp(n,q) and PSL(n,q) 
##  are isomorphic. 
##  The rang or n and q are as follows,
##        q<= 241 for n=2,
##        q<= 20  for n=4,
##        q<= 5   for n=6,
##        q<= 3   for n=8,
##        q = 2   for n=10.
##
##EXAMPLES
##> with(FiniteGroups):
##> ProjectiveSymplecticGroup(2,64);
#!<   permgroup of degree 65 with 2 generators
##<   permgroup(65,{[[3,64,62,60,58,57,55,53,51,10,48,47,45,43,42,40,38,36,8,33,31,4,28,27,25,23,21,6,18,16,14,12,65,63,61,59,11,56,54,52,50,49,5,46,44,9,41,39,37,35,34,32,30,29,7,26,24,22,20,19,17,15,13]],[[1,2,3],[6,9,7],[8,11,10],[12,18,58],[13,24,51],[14,54,20],[15,35,38],[16,40,33],[17,43,29],[19,59,65],[21,46,22],[23,63,57],[25,36,28],[26,53,64],[27,32,30],[31,56,55],[34,60,48],[37,61,44],[39,42,62],[41,52,49],[45,50,47]]})
##> ProjectiveSymplecticGroupOrder(4,3);
##< 25920 
##
##SEEALSO
##- "SymplecticGroup"
##- "ProjectiveSpecialLinearGroup"

ProjectiveSymplecticGroup := proc(n::posint,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
gn := `` || n || `,` || q;
if type(n,odd) then
     error"the dimension must be an even number":
end if;
if n = 2 then
     A := traperror( ProjectiveSpecialLinearGroups[gn] ):
else
     A := traperror( ProjectiveSymplecticGroups[gn] ):
end if;
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

ProjectiveSymplecticGroupOrder := proc(n::posint,q) 
local k; 
if not IsPrimePower(q) then                                                                
   error "second argument must be a prime power but got: %1", q fi; 
(1/gcd(2,q-1))*(q^((n/2)^2))*(product( (q^(2*k)-1), k=1..n/2) ); 
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[GeneralOrthogonalGroup]
##TITLE                 GeneralOrthogonalGroup
##TITLE                 GeneralOrthogonalGroupOrder
##ALIAS			FiniteGroups[GeneralOrthogonalGroup], FiniteGroups[GeneralOrthogonalGroupOrder]
##CALLINGSEQ
##- GeneralOrthogonalGroup('d', 'n', 'q')
##- GeneralOrthogonalGroupOrder('d', 'n', 'q')
##
##PARAMETERS
##- 'd'    :  0, 1 or -1
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the general orthogonal group GO(d,n,q).
##
##DESCRIPTION
##- The general orthogonal group is the set of all n x n matrices over the field 
##  with q elements that respect a non-singular quadratic form. The value of d 
##  must be 0 for odd n, of respectively 1 or -1 for even n. 
##  The range for n and q are as follows,
##        q<= 100 for n=2,
##        q<= 20  for n=3,
##        q<= 10  for n=4,
##        q<= 5   for n=5,
##        q = 3   for n=6, 7, 8,
##        q = 2   for n=9, 10, 11.
##
##EXAMPLES
##> with(FiniteGroups):
##> GeneralOrthogonalGroup(0,7,2);
#!<   permgroup of degree 64 with 2 generators
##<   permgroup(64,{[[5,6],[7,9],[8,11],[10,14],[12,16],[13,18],[15,21],[17,23],[19,26],[22,29],[24,31],[25,33],[27,36],[30,39],[34,43],[35,45],[38,47],[40,49],[42,52],[44,48],[50,54],[51,58],[55,61],[56,57]],[[2,3,4,5,7,10],[6,8,12,17,24,32],[9,13,19,27,37,47],[11,15,14,20,28,38],[16,22,23,30,40,50],[18,25,34,44,54,21],[26,35],[31,41,51,43,49,57],[33,42,53,60,61,45],[36,46,55],[39,48,56],[52,59,62,63,64,58]]})
##> GeneralOrthogonalGroup(1,4,5);
#!<   permgroup of degree 144 with 2 generators
##<   permgroup(144,{[[1,2,4,8],[3,6,9,5],[7,12,19,33],[10,15,25,45],[11,17,29,53],[13,21,37,65],[14,23,41,73],[16,27,49,84],[18,31,57,95],[20,35,62,104],[22,39,69,110],[24,43,76,118],[26,47,81,124],[28,51,56,93],[30,55,92,133],[32,59,48,82],[34,61,102,66],[36,63,44,78],[38,67,72,114],[40,71,113,138],[42,75,117,142],[46,80,123,85],[50,86,90,129],[52,89,128,115],[54,91,131,96],[58,97,101,103],[60,100,137,144],[64,106],[68,109,125,108],[70,111],[74,116,141,119],[77,120,122,132],[79,105,139,130],[83,99],[87,112,140,126],[88,94],[98,136,107,135],[121,127,134,143]],[[1,3,7,13,22,40,72,115,97,135,118,75,4,9,14,24,44,79,122,144,86,126,65,35],[2,5,10,16,28,52,90,130,67,108,95,55,8,6,11,18,32,60,101,138,120,143,84,47],[12,20,36,64,107,91,132,137,123,39,70,112,23,42,69,111,140,80,114,128,131,78,106,136],[15,26,48,83,125,61,103,113,141,51,88,127,17,30,56,94,134,116,129,139,102,59,99,109],[19,34,21,38,68,33,25,46,27,50,87,45,41,74,43,77,121,73,29,54,31,58,98,53],[37,66,104,92,63,105,57,96,133,117,93,89,76,119,142,81,110,71,49,85,124,62,82,100]]})
##> GeneralOrthogonalGroup(-1,4,5);
#!<   permgroup of degree 104 with 2 generators
##<   permgroup(104,{[[1,2,4,8],[3,6,12,23],[5,10,19,36],[7,14,27,45],[9,17,32,50],[11,21,40,59],[13,25,44,64],[15,29,35,22],[16,30,28,46],[18,34,54,76],[20,38,58,81],[24,42,61,70],[26,37,56,78],[31,49,71,90],[33,52,75,60],[39,51,73,91],[41,57,80,96],[43,63,86,66],[47,67,83,53],[48,69,85,77],[55,74,93,103],[62,84,94,104],[65,87,92,99],[68,89,101,72],[79,95,98,102],[82,97,88,100]],[[1,3,7,15],[2,5,11,22],[4,9,18,35],[6,13,26,45],[8,16,31,29],[10,20,39,59],[12,24,43,21],[14,28,47,68],[17,33,53,76],[19,37,57,34],[23,41,60,71],[25,38,52,69],[27,36,55,77],[30,48,70,90],[32,51,74,49],[40,50,72,64],[42,62,85,98],[44,65,56,79],[46,66,81,54],[58,82,73,92],[61,83,91,78],[63,87,99,103],[67,88,75,94],[80,97,100,101],[84,86,93,104],[89,102,95,96]]})
##> GeneralOrthogonalGroupOrder(0,7,3);
##< 18341406720 
##> GeneralOrthogonalGroupOrder(1,8,2);
##< 348364800 
##> GeneralOrthogonalGroupOrder(-1,8,2);
##< 394813440 
##
##SEEALSO
##- "SpecialOrthogonalGroup"
##- "GeneralLinearGroup"

GeneralOrthogonalGroup := proc(d,n,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "third argument must be a prime power but got: %1", q fi; 
gn := ``|| d || `,` || n || `,` || q;
A := traperror( GeneralOrthogonalGroups[gn] ):
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

GeneralOrthogonalGroupOrder := proc(d,n::posint,q) 
local k, s; 
if not member(d,{1,-1,0}) then                                                                
   error "first argument must be 1, -1 or 0 but got: %1", d fi;
if not IsPrimePower(q) then                                                                
  error "third argument must be a prime power but got: %1", q fi; 
if d=0 and type(n,odd) then
  s := gcd(2,q-1)*(q^(((n-1)/2)^2))*(product( (q^(2*k)-1), k=1..(n-1)/2) ); 
fi; 
if d=1 and type(n,even) then
  s := gcd(2,q-1)*gcd(2,q)*q^((n/2)*(n/2-1))*(q^(n/2)-1)*product( (q^(2*k)-1), k=1..(n/2-1) ); 
fi;
if d=-1 and type(n,even) then
  s :=gcd(2,q-1)*gcd(2,q)*q^((n/2)*(n/2-1))*(q^(n/2)+1)*product( (q^(2*k)-1), k=1..(n/2-1) ); 
fi;
if type(s,integer) then s;
else FAIL; fi;
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[SpecialOrthogonalGroup]
##TITLE                 SpecialOrthogonalGroup
##TITLE                 SpecialOrthogonalGroupOrder
##ALIAS			FiniteGroups[SpecialOrthogonalGroup], FiniteGroups[SpecialOrthogonalGroupOrder]
##CALLINGSEQ
##- SpecialOrthogonalGroup('d', 'n', 'q')
##- SpecialOrthogonalGroupOrder('d', 'n', 'q')
##
##PARAMETERS
##- 'd'    :  0, 1 or -1
##- 'n'    :  a positive integer
##- 'q'    :  power of a prime number
##
##RETURNS
##- A permutation representation of the special orthogonal group SO(d,n,q).
##
##DESCRIPTION
##- The special orthogonal group is the set of all n x n matrices over the field 
##  with q elements that respect a non-singular quadratic form with determinant 
##  one. The value of d must be 0 for odd n, of respectively 1 or -1 for 
##  even n. Note that for q even the groups SO(d,n,q) and GO(d,n,q) are isomorphic.   
##  The range for n and q are as follows,
##        q<= 100 for n=2,
##        q<= 20  for n=3,
##        q<= 10  for n=4,
##        q<= 5   for n=5,
##        q = 3   for n=6, 7, 8.
##
##EXAMPLES
##> with(FiniteGroups):
##> SpecialOrthogonalGroup(0,9,2);
#!<   permgroup of degree 256 with 2 generators
##<   permgroup(256,{[[3,4],[5,7],[6,9],[13,16],[15,19],[17,22],[18,24],[20,27],[21,29],[25,34],[26,36],[28,39],[33,45],[35,48],[37,51],[38,53],[40,56],[42,59],[46,63],[47,65],[50,69],[52,72],[54,75],[55,77],[58,81],[60,84],[62,87],[64,90],[68,80],[70,71],[73,97],[74,99],[82,107],[83,109],[85,112],[88,116],[89,118],[93,105],[94,122],[95,124],[96,126],[98,129],[106,138],[108,141],[110,144],[111,146],[113,130],[115,151],[117,154],[121,157],[123,160],[125,163],[127,166],[128,168],[132,173],[136,177],[137,178],[139,181],[140,183],[143,187],[145,165],[147,169],[148,170],[152,153],[158,195],[159,197],[162,201],[164,204],[167,208],[172,211],[176,213],[179,216],[180,192],[182,220],[188,225],[189,209],[191,224],[194,229],[196,232],[202,219],[203,218],[205,231],[206,238],[207,226],[212,235],[214,241],[215,242],[217,245],[221,236],[228,249],[230,246],[237,251],[240,254],[243,244],[247,248],[252,256]],[[2,3,5,8],[4,6,10,13,17,23,32,44],[7,11,14,18,25,35,49,68],[9,12,15,20,28,40,57,80],[16,21,30,42,60,85,113,149],[19,26,37,52,73,98,130,171],[22,31,43,61,86,114,150,191],[24,33,46,64,75,101,133,174],[27,38,54,76,102,134,175,212],[29,41,58,82,108,142,186,224],[34,47,66,92,120,156,193,228],[36,50,70,94,123,161,200,235],[39,55,78,104,136,157,194,230],[45,62,88,117,141,185,223,249],[48,67,93,121,158,196,216,244],[51,71,95,125,164,205,229,251],[53,74,100,132,138,180,218,246],[56,79,105,137,179,217,63,89],[59,83,110,145,188,226,197,233],[65,91,119,155,177,178,215,243],[69,72,96,127,167,163,203,237],[77,103,135,176,214,146,189,118],[81,106,139,182,221,122,159,198],[84,111,147,151,154,183,222,211],[87,115,152,192,204,232,242,245],[90,99,131,172,109,143,168,209],[97,128,169,116,153,181,219,247],[107,140,184,173],[112,148,190,227,250,255,256,241],[124,162,202,236],[126,165,206,225,208,201,220,248],[129,170,210,239,253,213,240,187],[144,166,207,160,199,234,252,254],[195,231]]})
##> SpecialOrthogonalGroup(1,4,7);
#!<   permgroup of degree 128 with 3 generators
##<   permgroup(128,{[[1,2],[3,5,11,25,59,17],[4,8,18,42,83,24],[6,13,30,67,90,100],[7,16,39,77,14,33],[9,20,47,89,68,82],[10,23,56,76,21,50],[12,28,65,34,71,69],[15,36,75,111,118,126],[19,45,87,51,64,91],[22,53,95,120,109,125],[26,61,80,55,31,32],[29,66,104,84,101,70],[35,73,108,58,99,103],[37,54],[38,48,49,43,72,98],[40,79,62,78,94,119],[41,81,113,52,93,114],[46,88,107,60,102,92],[57,97,85,96,74,110],[63,105,106,115,124,121],[86,116,117,122,128,112]],[[1,2],[3,6,14,34,61,104],[4,9,21,51,72,107],[5,12,29,55,77,94],[7,17,40,80,67,28],[8,19,46,38,76,74],[10,24,57,98,89,45],[11,26,62,33,70,90],[13,31,59,101,65,78],[15,37,75,109,125,111],[16,32,69,100,119,84],[18,43,85,50,92,68],[20,48,83,102,87,96],[22,54,95,118,126,120],[23,49,91,82,110,60],[25,39,30,66,79,71],[27,63,106,115,105,124],[36,53],[41,81,114,113,127,93],[42,56,47,88,97,64],[44,86,117,122,116,128],[58,99,108,103,123,73]],[[1,3,7,13,32,66,12],[2,4,10,20,49,88,19],[5,11,27,64,21,52,94],[6,15,38,42,84,113,28],[8,18,44,71,14,35,74],[9,22,55,25,60,103,45],[16,31,30,68,81,106,110],[17,41,82,115,29,54,53],[23,48,47,90,99,117,119],[24,58,100,122,46,37,36],[26,39,78,93,118,125,124],[33,65,51,61,79,105,114],[34,72,97,116,108,50,87],[40,80,112,111,123,101,70],[43,56,96,73,109,126,128],[57,98,121,120,127,102,92],[59,95,91,107,86,89,77],[63,67,76,83,75,69,104]]})
##> SpecialOrthogonalGroup(-1,4,7);
#!<   permgroup of degree 100 with 3 generators
##<   permgroup(100,{[[1,2],[3,5,11,25,57,40],[4,8,18,41,78,56],[6,13,30,68,86,22],[7,12,28,34,73,94],[9,20,46,84,70,15],[10,19,44,50,89,99],[14,33,17,39,38,76],[16,36,72,80,91,71],[21,49,24,55,54,60],[23,52,88,62,96,87],[26,59,93,37,31,69],[27,61,90,48,58,82],[29,45],[32,64,65,43,77,75],[35,74,83,100,63,92],[42,79,95,53,47,85],[51,67,66,97,81,98]],[[1,2],[3,6,14,34,59,72],[4,9,21,50,79,88],[5,7,16,37,76,65],[8,10,23,53,60,82],[11,26,32,33,71,86],[12,17,40,77,93,68],[13,31,57,91,28,64],[15,27,62,55,85,99],[18,42,48,49,87,70],[19,24,56,61,95,84],[20,47,78,96,44,58],[22,43,80,39,69,94],[25,38,30,36,75,73],[29,66,63,92,97,51],[35,45,83,81,98,100],[41,54,46,52,90,89],[67,74]],[[1,3,7,17],[2,4,10,24],[5,12,29,67],[6,15,35,39],[8,19,45,74],[9,22,51,55],[11,27,63,61],[13,32,36,69],[16,38,50,30],[18,43,81,77],[20,48,52,85],[23,54,34,46],[25,58,93,56],[26,60,59,94],[28,65,53,89],[31,70,75,80],[33,72,83,87],[37,73,44,82],[40,41,64,95],[42,76,79,99],[47,86,90,62],[49,88,66,71],[57,92,91,84],[68,78,98,96]]})
##> SpecialOrthogonalGroupOrder(0,7,3);
##< 9170703360 
##> SpecialOrthogonalGroupOrder(1,6,4);
##< 1974067200
##> SpecialOrthogonalGroupOrder(-1,6,4);
##< 2036736000 
##
##SEEALSO
##- "GeneralOrthogonalGroup"

SpecialOrthogonalGroup := proc(d,n,q)
local A, gn;
if not IsPrimePower(q) then                                                                
   error "third argument must be a prime power but got: %1", q fi; 
gn := ``|| d || `,` || n || `,` || q;
if type(q,even) then
     A := traperror( GeneralOrthogonalGroups[gn] ):
else
     A := traperror( SpecialOrthogonalGroups[gn] ):
end if;
if type(A,list) then permgroup(A[1],{op(2..nops(A),A)});
else FAIL;
end if;
end:

SpecialOrthogonalGroupOrder := proc(d,n::posint,q) 
local k, s; 
if not member(d,{1,-1,0}) then                                                                
   error "first argument must be 1, -1 or 0 but got: %1", d fi;
if not IsPrimePower(q) then                                                                
   error "third argument must be a prime power but got: %1", q fi; 
if d=0 and type(n,odd) then
   s := (q^(((n-1)/2)^2))*(product( (q^(2*k)-1), k=1..(n-1)/2) ); 
fi;
if d=1 and type(n,even) then
  s := gcd(2,q)*q^((n/2)*(n/2-1))*(q^(n/2)-1)*product( (q^(2*k)-1), k=1..(n/2-1) ); 
fi;
if d=-1 and type(n,even) then
   s := gcd(2,q)*q^((n/2)*(n/2-1))*(q^(n/2)+1)*product( (q^(2*k)-1), k=1..(n/2-1) ); 
fi;
if type(s,integer) then s;
else FAIL; fi;
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[AlternatingGroup]
##ALIAS			AlternatingGroup
##CALLINGSEQ
##- AlternatingGroup('n')
##
##PARAMETERS
##- 'n'    :  a positive integer
##
##RETURNS
##- A permutation representation of the alternating group An.
##
##DESCRIPTION
##- The alternating group on n element is the set of all even permutations 
##  on {1,...,n} for n >= 1. 
##
##EXAMPLES
##> with(FiniteGroups):
##> AlternatingGroup(7);
##<   permgroup(7, {[[5, 6, 7]], [[1, 2, 3, 4, 5, 6, 7]]})
##
##SEEALSO
##- "SymmetricGroup"

AlternatingGroup := proc(n::nonnegint)
local gen1, gen2;
if n = 2 or n = 1 then 
      return permgroup(n,{[]}); 
end if;
gen2 := [[n-2,n-1,n]];
if type(n,odd) then
      gen1 := [[seq(i,i=1..n)]];
else  gen1 := [[seq(i,i=1..n-1)]]; 
end if;
return permgroup(n,{gen1,gen2});
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[SymmetricGroup]
##ALIAS			SymmetricGroup
##CALLINGSEQ
##- SymmetricGroup('n')
##
##PARAMETERS
##- 'n'    :  a positive integer
##
##RETURNS
##- A permutation representation of the symmetric group Sn.
##
##DESCRIPTION
##- The symmetric group on n element is the set of all permutations on {1,...,n}
##  for n >= 0. 
##
##EXAMPLES
##> with(FiniteGroups):
##> SymmetricGroup(7);
##<   permgroup(7, {[[1, 2]], [[1, 2, 3, 4, 5, 6, 7]]})
##
##SEEALSO
##- "AlternatingGroup"

SymmetricGroup := proc(n::nonnegint)
local gen1, gen2;
if n=1 then 
     return permgroup(1,{[]}); 
end if;
gen1 := [[1,2]];
if n = 2 then 
     return permgroup(2,{gen1}); 
else gen2 := [[seq(i,i=1..n)]];
     return permgroup(n,{gen1,gen2});
end if;
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[MathieuGroup]
##TITLE                 MathieuGroup
##TITLE                 MathieuGroupOrder
##ALIAS			FiniteGroups[MathieuGroup], FiniteGroups[MathieuGroupOrder]
##CALLINGSEQ
##- MathieuGroup('n')
##- MathieuGroupOrder('n')
##
##PARAMETERS
##- 'n'    :  an integer in { 9, 10, 11, 12, 21, 22, 23, 24 }
##
##RETURNS
##- A permutation representation of the Mathieu group Mn.
##
##DESCRIPTION
##- The Mathieu group of degree n, where degree n must be in
##  { 9, 10, 11, 12, 21, 22, 23, 24 }. This is a sporadic group for 
##  n=11, 12, 22, 23, 24. 
##
##EXAMPLES
##> with(FiniteGroups):
##> MathieuGroup(11);
##<   permgroup(11, {[[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]], [[3, 7, 11, 8], [4, 10, 5, 6]]})
##> MathieuGroupOrder(10);
##< 720 
##
##SEEALSO
##- "SporadicGroup"

MathieuGroup := proc(n)
local A, gn;
gn := ``|| n;
A := traperror( MathieuGroups[gn] ):
if type(A,list) then permgroup(A[2],{op(3..nops(A),A)});
else FAIL;
end if;
end:

MathieuGroupOrder := proc(n)
local A, gn;
gn := ``|| n;
A := traperror( MathieuGroups[gn] ):
if type(A,list) then A[1];
else FAIL;
end if;
end:


############################################################################
##PROCEDURE(doti)	FiniteGroups[SporadicGroup]
##TITLE                 SporadicGroup
##TITLE                 SporadicGroupOrder
##ALIAS			FiniteGroups[SporadicGroup], FiniteGroups[SporadicGroupOrder]
##CALLINGSEQ
##- SporadicGroup(`name`)
##- SporadicGroupOrder(`name`)
##
##PARAMETERS
##- 'name'    :  a name in {J1, J2, J3, M11, M12, M22, M23, M24,
##                          HS, McL, HE, Co1, Co2, Co3, Suz, Fi22, Fi23, `Fi24'`, Ru, T, `O'N`}
##
##RETURNS
##- A permutation representation of sporadic groups:
##   M11, M12, M22, M23, M24   Mathieu groups
##   J1, J2, J3                Janko groups
##   HS                        Higman-Sims group
##   McL                       McLaughlin group
##   HE                        Held group
##   Co1, Co2, Co3             Conway groups
##   Suz                       Suzuki group
##   Fi22, Fi23, Fi24'         Fischer groups
##   Ru                        Rudvalis group
##   T                         Tits group
##   O'N                       O'Nan group
##
##DESCRIPTION 
##- The sporadic groups are the 26 finite simple groups that do not fit into 
##  any of the four infinite families of finite simple groups (i.e., the 
##  cyclic groups of prime order, alternating groups of degree at least five, 
##  Lie-type Chevalley groups, and Lie-type groups). 
##
##  The Tits group T is a finite simple group of order 17971200. It is the derived 
##  subgroup of the twisted Chevalley group 2^F_4(2). The Tits group itself is not a 
##  group of Lie type, and in fact it has sometimes been considered to be a sporadic 
##  group.
##
##EXAMPLES
##> with(FiniteGroups):
##> SporadicGroup(`M23`);
#!<   permgroup of degree 23 with 2 generators
##<   permgroup(23,{[[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23]],[[3,17,10,7,9],[4,13,14,19,5],[8,18,11,12,23],[15,20,22,21,16]]})
##> SporadicGroup(`Co3`);
#!<   permgroup of degree 276 with 2 generators
##<   permgroup(276,{[[1,245,185],[2,42,87],[3,112,266],[4,15,22],[5,131,30],[6,7,188],[8,75,111],[9,132,82],[12,187,124],[13,186,136],[14,265,213],[16,159,99],[17,256,130],[18,43,167],[19,101,31],[20,123,269],[21,134,74],[23,32,60],[24,209,67],[25,238,162],[26,35,154],[27,45,221],[28,235,270],[29,126,129],[33,66,210],[34,80,114],[36,251,229],[37,117,161],[38,206,63],[39,71,196],[40,118,180],[41,93,170],[44,271,164],[46,261,108],[47,182,49],[48,155,248],[50,230,153],[51,172,103],[52,236,165],[53,109,64],[54,191,100],[55,76,203],[56,156,260],[57,73,149],[58,116,145],[59,147,273],[61,127,189],[62,231,197],[65,122,169],[69,86,95],[70,255,192],[72,139,78],[77,252,151],[79,262,184],[81,214,242],[83,181,223],[84,174,200],[88,250,276],[89,257,244],[90,243,91],[92,158,107],[94,148,215],[96,105,125],[97,249,202],[98,263,193],[102,115,175],[106,219,150],[110,204,152],[113,225,216],[119,237,166],[120,241,234],[121,224,195],[128,190,268],[133,143,228],[135,168,201],[137,227,247],[138,217,240],[140,258,232],[141,220,205],[142,178,207],[144,146,254],[157,274,179],[160,253,176],[163,272,226],[171,233,194],[173,246,212],[177,198,211],[183,267,222],[199,208,259],[218,264,239]],[[1,204,123,82],[2,203,14,53],[3,33,40,118],[4,236,168,138],[6,172,188,157],[7,77,25,242],[8,76,85,264],[9,47,22,190],[10,146,50,26],[11,133,220,254],[12,224,179,58],[13,229,169,23],[15,84,148,78],[17,223],[18,228],[19,130,104,167],[20,131,90,60],[21,252,185,205],[24,263,214,81],[27,32,209,61],[28,196,67,137],[29,199,87,48],[30,65,218,112],[31,246,98,213],[34,99,165,265],[35,241],[36,198,161,89],[37,269],[38,251,42,271],[39,75,260,193],[41,176,192,57],[43,183,91,171],[44,116,173,102],[45,197,119,73],[46,238,124,162],[49,178,83,274],[51,174,244,100],[52,129],[54,153,217,151],[55,272,136,237],[56,257,154,121],[62,175,219,215],[63,106,66,259],[64,159,210,194],[68,258,221,234],[69,261,134,155],[70,164,211,266],[71,262],[72,189,114,222],[74,177,256,135],[79,226,267,202],[80,96,120,239],[86,132,216,160],[88,117,231,109],[92,201,111,276],[93,101,115,166],[94,253],[97,142,270,110],[105,243,212,225],[107,141,230,249],[113,139],[122,170,126,207],[125,163,184,158],[127,145,206,149],[140,273],[143,248,247,195],[144,180],[147,250,182,187],[152,232,208,191],[186,235,233,275]]})
##> SporadicGroupOrder(`McL`);
##< 898128000  
##
##SEEALSO
##- "MathieuGroup"

SporadicGroup := proc(s)
local A;
A := traperror( SporadicGroups[s] ):
if type(A,list) then permgroup(A[2],{op(3..nops(A),A)});
else FAIL;
end if;
end:

SporadicGroupOrder := proc(s)
local A;
A := traperror( SporadicGroups[s] ):
if type(A,list) then A[1];
else FAIL;
end if;
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[ExceptionalGroup]
##TITLE                 ExceptionalGroup
##TITLE                 ExceptionalGroupOrder
##ALIAS			FiniteGroups[ExceptionalGroup], FiniteGroups[ExceptionalGroupOrder]
##CALLINGSEQ
##- ExceptionalGroup(`name`)
##- ExceptionalGroupOrder(`name`)
##
##PARAMETERS
##- 'name'    :  a name in {`G2(2)`, `G2(3)`, `G2(4)`, `G2(5)`, `R(3)`, `R(27)`, `Sz(8)`, 
##                          `Sz(32)`, `3D4(2)`, `3D4(3)`, `F4(2)`}
##
##RETURNS
##- A permutation representation of exceptional groups G2(2), G2(3), G2(4), G2(5), 
##  R(3), R(27), Sz(8), Sz(32), 3D4(2), 3D4(3) and F4(2).
##
##DESCRIPTION 
##- Twisted or exceptional groups of Lie type are a class of finite simple groups.
##  These are the Chevalley groups G_2(q), Ree groups R(q), Suzuki groups Sz(q)
##  and Steinberg-Tits triality Groups ^3D_4(q), where q is a power of a prime.
##
##EXAMPLES
##> with(FiniteGroups):
##> ExceptionalGroup(`Sz(8)`);
#!<   permgroup of degree 65 with 2 generators
##<   permgroup(65,{[[1,2],[3,4],[5,7],[6,9],[8,12],[10,13],[11,15],[14,19],[16,21],[17,23],[18,25],[20,28],[22,31],[24,33],[26,35],[27,32],[29,37],[30,39],[34,43],[36,46],[38,48],[41,51],[42,44],[45,55],[47,50],[49,58],[52,60],[53,61],[54,59],[56,62],[57,63],[64,65]],[[1,3,5,8],[4,6,10,14],[7,11,16,22],[9,12,17,24],[13,18,26,36],[15,20,29,38],[19,27,31,28],[21,30,40,50],[23,32,41,52],[25,34,44,54],[33,42,53,43],[35,45,56,63],[37,47,51,46],[39,49,59,60],[48,57,55,58],[61,64,62,65]]})
##> ExceptionalGroupOrder(`3D4(2)`);
##< 211341312  
##
##SEEALSO
##- "OrthogonalGroup"

ExceptionalGroup := proc(s)
local A;
A := traperror( ExceptionalGroups[s] ):
if type(A,list) then permgroup(A[2],{op(3..nops(A),A)});
else FAIL;
end if;
end:

ExceptionalGroupOrder := proc(s)
local A;
A := traperror( ExceptionalGroups[s] ):
if type(A,list) then A[1];
else FAIL;
end if;
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[OrthogonalGroup]
##TITLE                 OrthogonalGroup
##TITLE                 OrthogonalGroupOrder
##ALIAS			FiniteGroups[OrthogonalGroup], FiniteGroups[OrthogonalGroupOrder]
##CALLINGSEQ
##- OrthogonalGroup(`name`)
##- OrthogonalGroupOrder(`name`)
##
##PARAMETERS
##- 'name'    :  a name in {`O7(3)`, `O8-(2)`, `O8+(2)`, `O8-(3)`, `O8+(3)`, `O10-(2)`, 
##                          `O10+(2)`}
##
##RETURNS
##- A permutation representation of orthogonal groups G2(4), G2(5), R(27), 
##  Sz(8), Sz(32) and 3D4(2).
##
##DESCRIPTION 
##- Orthogonal groups is a class of finite simple groups of Lie type.
##
##EXAMPLES
##> with(FiniteGroups):
##> OrthogonalGroup(`O8+(2)`);
#!<   permgroup of degree 120 with 2 generators
##<   permgroup(120,{[[1,2],[3,4],[5,7],[6,9],[8,12],[10,15],[11,17],[13,20],[14,21],[16,24],[18,26],[19,28],[22,32],[23,34],[25,37],[27,39],[29,41],[31,43],[33,45],[35,47],[36,48],[38,50],[40,53],[42,54],[44,57],[46,60],[49,63],[51,64],[56,68],[58,71],[59,73],[61,76],[62,78],[65,81],[66,82],[67,84],[70,86],[72,89],[74,90],[75,79],[77,92],[80,96],[83,98],[85,93],[87,101],[88,95],[91,102],[94,104],[97,107],[99,106],[100,105],[103,110],[108,115],[109,116],[112,113],[114,119]],[[2,3,5,8,13],[4,6,10,16,25],[7,11,18,27,40],[9,14,22,33,17],[12,19,29,42,55],[15,23,35,20,30],[21,31,44,58,72],[24,36,39,52,65],[26,38,51,54,67],[34,46,61,77,93],[37,49,53,66,83],[43,56,69,60,75],[45,59,74,78,94],[48,62,57,70,87],[63,79,95,105,112],[64,80,97,104,111],[68,85,100,109,117],[71,88,102,110,118],[76,91,98,81,86],[82,84,99,108,92],[90,103,96,106,113],[107,114,116,115,120]]})
##> OrthogonalGroupOrder(`O10-(2)`);
##< 25015379558400   
##
##SEEALSO
##- "ExceptionalGroup"

OrthogonalGroup := proc(s)
local A;
A := traperror( OrthogonalGroups[s] ):
if type(A,list) then permgroup(A[2],{op(3..nops(A),A)});
else FAIL;
end if;
end:

OrthogonalGroupOrder := proc(s)
local A;
A := traperror( OrthogonalGroups[s] ):
if type(A,list) then A[1];
else FAIL;
end if;
end:

############################################################################
##PROCEDURE(doti)	FiniteGroups[CyclicGroup]
##ALIAS			CyclicGroup
##CALLINGSEQ
##- CyclicGroup('n')
##- CyclicGroup('n', 's')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 's'    :  equation of the form `style`=`grelgroup` or `permgroup` (optional)
##
##RETURNS
##- A representation of the cyclic group by generators and relations or by
##  permutations.
##
##DESCRIPTION
##- The cyclic group is an abelian group generated by a single element. 
##
##EXAMPLES
##> with(FiniteGroups):
##> CyclicGroup(14);
##<   grelgroup({a}, {[a, a, a, a, a, a, a, a, a, a, a, a, a, a]})
##> CyclicGroup(14, style=grelgroup);
##<   grelgroup({a}, {[a, a, a, a, a, a, a, a, a, a, a, a, a, a]})
##> CyclicGroup(14, style=permgroup);
##<   permgroup(14, {a = [[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]]})
##
##SEEALSO
##- "DicyclicGroup"
##- "MetacyclicGroup"

CyclicGroup := proc(n::nonnegint,s::`=`)
 if nargs=1 then CyclicGrelgroup(n);
 elif nargs=2 then
     if s=('style'='permgroup') then CyclicPermgroup(n);
     elif s=('style'='grelgroup') then CyclicGrelgroup(n);
     else error
          "second argument must be 'style'='permgroup' or 'style'='grelgroup'";
     end if;
 else error
      "must be 1 or 2 arguments";
 end if;
end proc:

###############

CyclicPermgroup := proc(n::nonnegint)
 local p,s;
 global a;
 a := evaln(a); 
 p := CyclicGrelgroup(n);
 s := subgrel({y=[seq(a,i=1..n)]},p);
 group:-permrep(s);
end proc:

###############

CyclicGrelgroup := proc(n::nonnegint)
 local i;
 global a;
 a := evaln(a);
 grelgroup({a},{[seq(a,i=1..n)]});
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[DihedralGroup]
##ALIAS			DihedralGroup
##CALLINGSEQ
##- DihedralGroup('n')
##- DihedralGroup('n', 's')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 's'    :  equation of the form `style`=`grelgroup` or `permgroup` (optional)
##
##RETURNS
##- A representation of the dihedral group by generators and relations or by
##  permutations.
##
##DESCRIPTION
##- The dihedral group is the symmetry group of an n-sided regular polygon for 
##  n > 1.
##
##EXAMPLES
##> with(FiniteGroups):
##> DihedralGroup(13);
##<   grelgroup({a, b}, {[b, b], [a, b, a, b], [a, a, a, a, a, a, a, a, a, a, a, a, a]})
##> DihedralGroup(13, style=grelgroup);
##<   grelgroup({a, b}, {[b, b], [a, b, a, b], [a, a, a, a, a, a, a, a, a, a, a, a, a]})
##> DihedralGroup(13, style=permgroup);
##<   permgroup(13, {[[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13]], [[1, 12], [2, 11], [3, 10], [4, 9], [5, 8], [6, 7]]})
##
##SEEALSO
##- "DicyclicGroup"

DihedralGroup := proc(n::nonnegint,s::`=`)
 if nargs=1 then DihedralGrelgroup(n);
 elif nargs=2 then
     if s=('style'='permgroup') then DihedralPermgroup(n);
     elif s=('style'='grelgroup') then DihedralGrelgroup(n);
     else error
          "second argument must be 'style'='permgroup' or 'style'='grelgroup'";
     end if;
 else error
      "must be 1 or 2 arguments";
 end if;
end proc:

###############

DihedralPermgroup := proc(n::nonnegint)
 local i;
 if n<=2 then error "must have n>2"; end if;
 permgroup(n,{[[seq(i,i=1..n)]],[seq([i,n-i],i=1..iquo(n-1,2))]});
end proc:

###############

DihedralGrelgroup := proc(n::nonnegint)
 local i;
 global a,b;
 if n<=2 then error "must have n>2"; end if;
 a := evaln(a); b := evaln(b);
 grelgroup({a,b},{[seq(a,i=1..n)],[b,b],[a,b,a,b]});
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[DicyclicGroup]
##ALIAS			DicyclicGroup
##CALLINGSEQ
##- DicyclicGroup('n')
##- DicyclicGroup('n', 's')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 's'    :  equation of the form `style`=`grelgroup` or `permgroup` (optional)
##
##RETURNS
##- A representation of the dicyclic group by generators and relations or by
##  permutations.
##
##DESCRIPTION
##- The dicyclic group is a non-abelian group of order 4n which contains a 
##  cyclic subgroup of size 2n for n > 1.
##
##EXAMPLES
##> with(FiniteGroups):
##> DicyclicGroup(6);
##<   grelgroup({a, b}, {[1/b, a, b, a], [a, a, a, a, a, a, a, a, a, a, a, a], [a, a, a, a, a, a, b, b]})
##> DicyclicGroup(6, style=grelgroup);
##<   grelgroup({a, b}, {[1/b, a, b, a], [a, a, a, a, a, a, a, a, a, a, a, a], [a, a, a, a, a, a, b, b]})
##> DicyclicGroup(6, style=permgroup);
##<   permgroup(24,{a = [[1, 7, 8, 9, 10, 11, 3, 12, 13, 14, 15, 6], [2, 17, 18, 19, 20, 21, 4, 5, 22, 23, 24, 16]], b = [[1, 2, 3, 4], [5, 6, 17, 11], [7, 16, 12, 21], [8, 24, 13, 20], [9, 23, 14, 19], [10, 22, 15, 18]]})
##
##SEEALSO
##- "CyclicGroup"
##- "MetacyclicGroup"

DicyclicGroup := proc(n::nonnegint,s::`=`)
 if nargs=1 then DicyclicGrelgroup(n);
 elif nargs=2 then
     if s=('style'='permgroup') then DicyclicPermgroup(n);
     elif s=('style'='grelgroup') then DicyclicGrelgroup(n);
     else error
          "second argument must be 'style'='permgroup' or 'style'='grelgroup'";
     end if;
 else error
      "must be 1 or 2 arguments";
 end if;
end proc:

###############

DicyclicPermgroup := proc(n::nonnegint)
 local p,s;
 global a,b;
 if n<=2 then error "must have n>2"; end if;
 a := evaln(a); b := evaln(b);
 p := DicyclicGrelgroup(n);
 s := subgrel({y=[b,b,b,b]},p);
 group:-permrep(s);
end proc:

###############

DicyclicGrelgroup := proc(n::nonnegint)
 local i;
 global a,b;
 if n<=2 then error "must have n>2"; end if;
 a := evaln(a); b := evaln(b);
 grelgroup({a,b},{[seq(a,i=1..2*n)],[seq(a,i=1..n),b,b],[1/b,a,b,a]});
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[MetacyclicGroup]
##ALIAS			MetacyclicGroup
##CALLINGSEQ
##- MetacyclicGroup('m', 'n', 'l')
##- MetacyclicGroup('m', 'n', 'l', 's')
##
##PARAMETERS
##- 'm'    :  a positive integer
##- 'n'    :  a positive integer
##- 'l'    :  a positive integer
##- 's'    :  equation of the form `style`=`grelgroup` or `permgroup` (optional)
##
##RETURNS
##- A representation of the metacyclic group by generators and relations or by
##  permutations.
##
##DESCRIPTION
##- The dicyclic group is a group which has a cyclic normal subgroup such that 
##  its quotient group is also cyclic.
##
##EXAMPLES
##> with(FiniteGroups):
##> MetacyclicGroup(6, 8, 5);
##<   grelgroup({a, b}, {[a, a, a, a, a, a], [1/b, a, b, a, a, a, a, a], [b, b, b, b, b, b, b, b]})
##> MetacyclicGroup(6, 8, 5, style=grelgroup);
##<   grelgroup({a, b}, {[a, a, a, a, a, a], [1/b, a, b, a, a, a, a, a], [b, b, b, b, b, b, b, b]})
##> MetacyclicGroup(6, 8, 5, style=permgroup);
#! <   permgroup(48,{a=[[1,10,11,12,13,14],[2,15,16,17,18,19],[3,20,21,22,23,24],[4,25,26,27,28,29],[5,30,31,32,33,34],[6,35,36,37,38,39],[7,40,41,42,43,44],[8,9,45,46,47,48]],b=[[1,2,3,4,5,6,7,8],[9,10,15,20,25,30,35,40],[11,16,21,26,31,36,41,45],[12,17,22,27,32,37,42,46],[13,18,23,28,33,38,43,47],[14,19,24,29,34,39,44,48]]})
#! Don't include the output because the group package uses different
#! elements of the group to convert to a permutation representation of the
#! given group which results in different (but isomorphic) outputs.
##
##SEEALSO
##- "CyclicGroup"
##- "DicyclicGroup"

MetacyclicGroup := proc(n::nonnegint,m::nonnegint,l::nonnegint,s::`=`)
 if nargs=3 then MetacyclicGrelgroup(n,m,l);
 elif nargs=4 then
     if s=('style'='permgroup') then MetacyclicPermgroup(n,m,l);
     elif s=('style'='grelgroup') then MetacyclicGrelgroup(n,m,l);
     else error
          "Fourth argument must be 'style'='permgroup' or 'style'='grelgroup'";
     end if;
 else error
      "must be 3 or 4 arguments";
 end if;
end proc:

###############

MetacyclicPermgroup := proc(n::nonnegint,m::nonnegint,l::nonnegint)
 local p,s,i;
 global a,b;
 if n<=2 then error "must have n>2"; end if;
 a := evaln(a); b := evaln(b);
 p := MetacyclicGrelgroup(n,m,l);
 s := subgrel({y=[seq(b,i=1..m)]},p);
 group:-permrep(s);
end proc:

###############

MetacyclicGrelgroup := proc(n::nonnegint,m::nonnegint,l::nonnegint)
 local i;
 global a,b;
 if n<=2 then error "must have n>2"; end if;
 a := evaln(a); b := evaln(b);
 grelgroup({a,b},{[seq(a,i=1..n)],[seq(b,i=1..m)],[1/b,a,b,seq(a,i=1..l)]});
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[QuaternionGroup]
##ALIAS			QuaternionGroup
##CALLINGSEQ
##- QuaternionGroup()
##- QuaternionGroup('s')
##
##PARAMETERS
##- 's'    :  equation of the form `style`=`grelgroup` or `permgroup` (optional)
##
##RETURNS
##- A representation of the Quaternion group by generators and relations or by
##  permutations.
##
##DESCRIPTION
##- The quaternion group is a non-abelian group of order 8.
##
##EXAMPLES
##> with(FiniteGroups):
##> QuaternionGroup();
##<   grelgroup({i, j}, {[i, i, i, i], [i, i, j, j], [i, j, 1/i, j]})
##> QuaternionGroup(style=grelgroup);
##<   grelgroup({i, j}, {[i, i, i, i], [i, i, j, j], [i, j, 1/i, j]})
##> QuaternionGroup(style=permgroup);
##<   permgroup(8, {[[1, 5, 3, 8], [2, 7, 4, 6]], [[1, 2, 3, 4], [5, 6, 8, 7]]})
##

QuaternionGroup := proc(s::`=`)
 if nargs=0 then QuaternionGroupGrelgroup;
 elif nargs=1 then
     if s=('style'='permgroup') then QuaternionGroupPermgroup;
     elif s=('style'='grelgroup') then QuaternionGroupGrelgroup;
     else error
          "first argument must be 'style'='permgroup' or 'style'='grelgroup'";
     end if;
 else error
      "must be 0 or 1 arguments";
 end if;
end proc:

####################

QuaternionGroupPermgroup := permgroup(8,{[[1,2,3,4],[5,6,8,7]],[[1,5,3,8],[2,7,4,6]]}):
QuaternionGroupGrelgroup := grelgroup({i,j},{[i,i,i,i],[i,i,j,j],[i,j,1/i,j]}):

############################################################################
##PROCEDURE(doti)	FiniteGroups[SmallGroup]
##ALIAS			SmallGroup
##CALLINGSEQ
##- SmallGroup('n', 'd')
##- SmallGroup('n', 'd', 's')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 'd'    :  a positive integer
##- 's'    :  equation of the form `style`=`grelgroup` or `permgroup` (optional)
##
##RETURNS
##- The d-th small group of size n by generators and relations or by
##  permutations.
##
##DESCRIPTION
##- The small groups library contains all groups of small orders up to 200. 
##  The groups are sorted by their orders and they are listed up to isomorphism; 
##  that is, for each of the available orders a complete and irredundant list of 
##  isomorphism type representatives of groups is given. These groups are 
##  available as permutation groups and groups by generators and relations.
##
##EXAMPLES
##> with(FiniteGroups):
##> SmallGroup(6, 2);
##<   grelgroup({_a1, _a2},{[_a1, _a1], [_a2, _a2, _a2], [1/_a2, 1/_a1, _a2, _a1]})
##> SmallGroup(6, 2, style=grelgroup);
##<   grelgroup({_a1, _a2},{[_a1, _a1], [_a2, _a2, _a2], [1/_a2, 1/_a1, _a2, _a1]})
##> SmallGroup(6, 2, style=permgroup);
##<   permgroup(6, {_a1 = [[1, 2], [3, 6], [4, 5]], _a2 = [[1, 3, 4], [2, 6, 5]]})
##
##SEEALSO
##- "AllSmallGroups"
##- "NumSmallGroups"

SmallGroup := proc(n,d,s)
 if nargs = 2 then SmallGrelgroup(n,d);
 elif nargs = 3 then
     if s=('style'='permgroup') then SmallPermgroup(n,d);
     elif s=('style'='grelgroup') then SmallGrelgroup(n,d);
     else error
          "third argument must be 'style'='permgroup' or 'style'='grelgroup'";
     end if;
 else error
      "must be 2 or 3 arguments";
 end if;
end proc:

###############

SmallGrelgroup := proc(n,d)
local s, gn;
gn := ``|| n;
if n = 1 and d = 1 then 
   return grelgroup({a1},{[a1]});
end if;
s := traperror( SmallGroups[gn] ):
if n>200 or d>nops(s) then 
   return FAIL;
else
   return grelgroup({op(s[d][1])},{op(2..nops(s[d]),s[d])});
end if;
end proc:

###############

SmallPermgroup := proc(n,d)
local s, g, sg, sd, G, x, gn;
if n=1 and d=1 then 
   return permgroup(n,{[]});
end if;
gn := ``|| n;
s := traperror( SmallGroups[gn] ):
if n>200 or d>nops(s) then 
   return FAIL;
end if;
sd := s[d];
g := SmallGrelgroup(n,d):
          sg := subgrel({x=[]},g):
          return group:-permrep(sg);
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[AllSmallGroups]
##ALIAS			AllSmallGroups
##CALLINGSEQ
##- AllSmallGroups('n')
##- AllSmallGroups('n', 's')
##
##PARAMETERS
##- 'n'    :  a positive integer
##- 's'    :  equation of the form `style`=`grelgroup` or `permgroup` (optional)
##
##RETURNS
##- All the small groups of size n by generators and relations or by
##  permutations.
##
##DESCRIPTION
##- The small groups library contains all groups of small orders up to 200. 
##  The groups are sorted by their orders and they are listed up to isomorphism; 
##  that is, for each of the available orders a complete and irredundant list of 
##  isomorphism type representatives of groups is given. These groups are 
##  available as permutation groups and groups by generators and relations.
##
##EXAMPLES
##> with(FiniteGroups):
##> AllSmallGroups(6);
##<   [grelgroup({_a1, _a2},{[_a1, _a1], [1/_a2, 1/_a1, _a2, _a1, 1/_a2], [_a2, _a2, _a2]}), grelgroup({_a1, _a2},{[_a1, _a1], [_a2, _a2, _a2], [1/_a2, 1/_a1, _a2, _a1]})]
##> AllSmallGroups(6, style=grelgroup);
##<   [grelgroup({_a1, _a2},{[_a1, _a1], [1/_a2, 1/_a1, _a2, _a1, 1/_a2], [_a2, _a2, _a2]}), grelgroup({_a1, _a2},{[_a1, _a1], [_a2, _a2, _a2], [1/_a2, 1/_a1, _a2, _a1]})]
##> AllSmallGroups(6, style=permgroup);
##<   [permgroup(6,{_a1 = [[1, 2], [3, 4], [5, 6]], _a2 = [[1, 6, 3], [2, 4, 5]]}), permgroup(6,{_a1 = [[1, 2], [3, 6], [4, 5]], _a2 = [[1, 3, 4], [2, 6, 5]]})]
##
##SEEALSO
##- "SmallGroup"
##- "NumSmallGroups"

AllSmallGroups := proc(n,s)
 if nargs=1 then AllSmallGrelgroups(n);
 elif nargs=2 then
     if s=('style'='permgroup') then AllSmallPermgroups(n);
     elif s=('style'='grelgroup') then AllSmallGrelgroups(n);
     else error
          "second argument must be 'style'='permgroup' or 'style'='grelgroup'";
     end if;
 else error
      "must be 1 or 2 arguments";
 end if;
end proc:

###############

AllSmallGrelgroups := proc(n)
local a, s, gn;
if n>200 then return FAIL; end if;
gn := ``|| n;
s := traperror( SmallGroups[gn] ):
a := [seq(grelgroup({op(s[i][1])},{op(2..nops(s[i]),s[i])}),i=1..nops(s))];
return a;
end proc:

###############

AllSmallPermgroups := proc(n)
local a, s, ns, d, sg, g, x, gn;
a := [];
if n>200 then return FAIL; end if;
gn := ``|| n;
s := traperror( SmallGroups[gn] ):
ns := nops(s);
for d to ns do
   g := grelgroup({op(s[d][1])},{op(2..nops(s[d]),s[d])});
   sg := subgrel({x=[]},g):
   a := [op(a), group:-permrep(sg)];
end do;
return a;
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[NumSmallGroups]
##ALIAS			NumSmallGroups
##CALLINGSEQ
##- NumSmallGroups('n')
##
##PARAMETERS
##- 'n'    :  a positive integer
##
##RETURNS
##- the number of all non-isomorphic small groups of size n 
##
##EXAMPLES
##> with(FiniteGroups):
##> NumSmallGroups(128);
##<   2328
##
##SEEALSO
##- "SmallGroup"
##- "AllSmallGroups"

NumSmallGroups := proc(n)
local s, gn;
if n>200 then return FAIL; end if;
if n=1 then return n; end if;
gn := ``|| n;
s:=SmallGroups[gn]:
return nops(s);
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[MovedPoints]
##ALIAS			MovedPoints
##CALLINGSEQ
##- MovedPoints('g')
##
##PARAMETERS
##- 'g'    :  a permutation group
##
##RETURNS
##- the set of all numbers moved by a permutation group 
##
##EXAMPLES
##> with(FiniteGroups):
##> g := permgroup(8, {[[5, 6], [2, 4]], [[2, 4, 5, 6]]}):
##> MovedPoints(g);
##<   {2, 4, 5, 6}
##
##SEEALSO
##- "LargestMovedPoint"
##- "SmallestMovedPoint"
##- "FixedPoints"
##

MovedPoints := proc(g::permgroup)
local l, n, i, k, j;
n:={};
l:=op(2,g);
for i in l do
   if type(i, `=`) then k := op(2,i); else k := i; end if;
   for j in k do
      n:= n union {op(j)};
   end do;
end do;
n;
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[LargestMovedPoint]
##ALIAS			LargestMovedPoint
##CALLINGSEQ
##- LargestMovedPoint('g')
##
##PARAMETERS
##- 'g'    :  a permutation group
##
##RETURNS
##- the largest number moved by a permutation group 
##
##EXAMPLES
##> with(FiniteGroups):
##> g := permgroup(8, {[[5, 6], [2, 4]], [[2, 4, 5, 6]]}):
##> LargestMovedPoint(g);
##<   6
##
##SEEALSO
##- "SmallestMovedPoint"
##- "MovedPoints"
##- "FixedPoints"
##

LargestMovedPoint := proc(g::permgroup)
local l, n, i, k, j;
l:=op(2,g);
n := 0;
for i in l do
   if type(i, `=`) then k := op(2,i); else k := i; end if;
   for j in k do
      n:=max(n,op(j));
   end do;
end do;
n;
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[SmallestMovedPoint]
##ALIAS			SmallestMovedPoint
##CALLINGSEQ
##- SmallestMovedPoint('g')
##
##PARAMETERS
##- 'g'    :  a permutation group
##
##RETURNS
##- the smallest number moved by a permutation group 
##
##EXAMPLES
##> with(FiniteGroups):
##> g := permgroup(8, {[[5, 6], [2, 4]], [[2, 4, 5, 6]]}):
##> SmallestMovedPoint(g);
##<   2
##
##SEEALSO
##- "LargestMovedPoint"
##- "MovedPoints"
##- "FixedPoints"
##

SmallestMovedPoint := proc(g::group)
local l, n, i, k, j;
l:=op(2,g);
n := infinity;
for i in l do
   if type(i, `=`) then k := op(2,i); else k := i; end if;
   for j in k do
      n:=min(n,op(j));
   end do;
end do;
n;
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[FixedPoints]
##ALIAS			FixedPoints
##CALLINGSEQ
##- FixedPoints('g')
##
##PARAMETERS
##- 'g'    :  a permutation group
##
##RETURNS
##- the set of all numbers fixed by a permutation group 
##
##EXAMPLES
##> with(FiniteGroups):
##> g := permgroup(8, {[[5, 6], [2, 4]], [[2, 4, 5, 6]]}):
##> FixedPoints(g);
##<   {1, 3, 7, 8}
##
##SEEALSO
##- "LargestMovedPoint"
##- "SmallestMovedPoint"
##- "MovedPoints"
##

FixedPoints := proc(g::permgroup)
local f;
f:= {seq(i,i=1..op(1,g))} minus MovedPoints(g);
return f;
end proc:

############################################################################
##PROCEDURE(doti)	FiniteGroups[IsomorphismPermGroup]
##ALIAS			IsomorphismPermGroup
##CALLINGSEQ
##- IsomorphismPermGroup('g')
##
##PARAMETERS
##- 'g'    :  a group with relations and generators
##
##RETURNS
##- the isomorphism permutation group 
##
##EXAMPLES
##> with(FiniteGroups):
##> g := grelgroup({a, b}, {[b, b, b, b], [a, a, a], [1/b, a, b, a, a]}):
##> IsomorphismPermGroup(g);
##<   permgroup(12, {a = [[1, 5, 6], [2, 8, 9], [3, 10, 11], [4, 7, 12]], b = [[1, 2, 3, 4], [5, 8, 10, 7], [6, 9, 11, 12]]})
##

IsomorphismPermGroup:= proc(g::group)
group:-permrep(subgrel({x=[]},g));
end proc:

end module:


read"ExceptionalGroups.mpl": 
read"GeneralLinearGroups.mpl":
read"GeneralOrthogonalGroups.mpl":
read"GeneralUnitaryGroups.mpl":
read"MathieuGroups.mpl":
read"OrthogonalGroups.mpl":
read"ProjectiveGeneralLinearGroups.mpl":  
read"ProjectiveGeneralUnitaryGroups.mpl":
read"ProjectiveSpecialLinearGroups.mpl":
read"ProjectiveSpecialUnitaryGroups.mpl":
read"ProjectiveSymplecticGroups.mpl":
read"SpecialLinearGroups.mpl": 
read"SpecialOrthogonalGroups.mpl":
read"SpecialUnitaryGroups.mpl":
read"SporadicGroups.mpl":
read"SymplecticGroups.mpl":
read"SmallGroups.mpl":

#savelib(FiniteGroups):



